<?php
/*
Plugin Name:Google Merchant Center Variable Product Schema Fix for WooCommerce
Plugin URI: https://qwebmaster.com/google-merchant-center-variable-product-schema-fix-for-woocommerce
Description: This plugin addresses issues with Google Merchant Center specifically for "Mismatched value (page crawl) [price]" errors due to variable product schema markups. It allows enabling a fix globally or per product to ensure correct price display for variable products.
Version: 1.0.0
WC requires at least: 4.0
WC tested up to: 8.3.0
Author: Qwebmaster
Author URI: https://qwebmaster.com/
Copyright: © 2023 Qwebmaster
License:         GNU General Public License v3.0
License URI:     http://www.gnu.org/licenses/gpl-3.0.html
*/

// Add section to WooCommerce settings
add_filter( 'woocommerce_get_sections_products', 'qweb_add_gmc_fix_settings_section' );
function qweb_add_gmc_fix_settings_section( $sections ) {
    $sections['gmc_fix'] = __('Google Merchant Center Fix', 'woocommerce');
    return $sections;
}

add_filter('woocommerce_get_settings_products', 'qweb_add_gmc_fix_settings', 10, 2);
function qweb_add_gmc_fix_settings($settings, $current_section) {
    if ('gmc_fix' === $current_section) {
        $settings_gmc_fix = array();

        // Add title to the settings
        $settings_gmc_fix[] = array(
            'name' => __('Google Merchant Center Fix Settings', 'woocommerce'),
            'type' => 'title',
            'desc' => __('This fix addresses issues with Google Merchant Center where "Mismatched value (page crawl) [price]" errors occur. Enabling this option ensures that schema markup for variable products will not show a price range, which aligns with Googles requirements.', 'woocommerce'),
            'id' => 'gmc_fix_options'
        );

        // Add checkbox to enable or disable the fix globally
        $settings_gmc_fix[] = array(
            'name'     => __('Enable Fix Globally', 'woocommerce'),
            'desc_tip' => __('Enable this option to apply the Google Merchant Center fix to all variable products on your site.', 'woocommerce'),
            'id'       => 'gmc_fix_global_enable',
            'type'     => 'checkbox',
            'css'      => 'min-width:300px;',
            'desc'     => __('Apply fix to all variable products', 'woocommerce'),
        );



        // Add end section
        $settings_gmc_fix[] = array(
            'type' => 'sectionend',
            'id' => 'gmc_fix_options'
        );

        return $settings_gmc_fix;
    }

    return $settings;
}

add_action( 'woocommerce_product_options_general_product_data', 'qweb_add_custom_checkbox_to_products' );
function qweb_add_custom_checkbox_to_products() {
    global $post;

    // Get the product object
    $product_factory = new WC_Product_Factory();
    $product = $product_factory->get_product($post->ID);

    if ( $product && $product->is_type( 'variable' ) ) {
        woocommerce_wp_checkbox( array( 
            'id' => '_qwebmaster_wc_schema_checkbox', 
            'label' => __('Merchant Center Fix', 'woocommerce' ), 
            'description' => __( 'Check this box to use enable Google Merchant Center Fix for this product.', 'woocommerce' )
        ));

        // Explanatory text
        echo '<div class="variations-explanation" style="margin-top: 20px;">';
        echo '<p>' . __('Fixes an issue with Google Merchant Center where "Mismatched value (page crawl) [price]" errors occur. This is due to schema markup for variable products showing a price range.', 'woocommerce') . '</p>';
        echo '<p>' . __('Use the following permalinks for each variable product in the Google Merchant Center to ensure the correct price for each variant is displayed, or if you are using a plugin to upload the product data it should be modified to use the attribute links for the products as in the examples below. :', 'woocommerce') . '</p>';
        echo '</div>';

        // Display permalinks for each variation
        echo '<div class="variations-permalinks" style="background-color: #f7f7f7;border: 1px solid #dcdcdc;padding: 10px;border-radius: 4px;margin: 14px 20px;">';
        echo '<p style="font-weight: bold; margin-bottom: 5px;">' . __('Variation Permalinks:', 'woocommerce') . '</p>';
        
        $variations = $product->get_children();
        if ( ! empty( $variations ) ) {
            echo '<ul style="list-style: none; margin: 0; padding: 0;">';
            foreach ( $variations as $variation_id ) {
                $variation_permalink = get_permalink( $variation_id );
                echo '<li style="margin-bottom: 5px;"><a href="' . esc_url( $variation_permalink ) . '" target="_blank" style="text-decoration: none; color: #0071a1;">' . esc_html( $variation_permalink ) . '</a></li>';
            }
            echo '</ul>';
        } else {
            echo '<p>' . __('No variations found.', 'woocommerce') . '</p>';
        }

        echo '</div>';
    }
}


add_action( 'woocommerce_admin_process_product_object', 'qweb_save_custom_checkbox_to_products', 10, 1 );
function qweb_save_custom_checkbox_to_products( $product ) {
    $custom_schema = isset( $_POST['_qwebmaster_wc_schema_checkbox'] ) ? 'yes' : 'no';
    $product->update_meta_data( '_qwebmaster_wc_schema_checkbox', $custom_schema );
}

// Customize the product schema markup
add_filter('woocommerce_structured_data_product', 'qweb_customize_product_schema_markup', 10, 2);
function qweb_customize_product_schema_markup($markup, $product) {
    // Check the global setting first
    $global_schema_enabled = get_option('gmc_fix_global_enable', 'no');

    // Check if the custom schema checkbox is enabled for this product
    $custom_schema_enabled = ('yes' === $global_schema_enabled) ? 'yes' : $product->get_meta('_qwebmaster_wc_schema_checkbox', true);

    if ($custom_schema_enabled === 'yes' && $product->is_type('variable')) {
        $variation_id = qweb_find_matching_product_variation_id($product);

        if ($variation_id) {
            $variation = new WC_Product_Variation($variation_id);

            // Modify the offers field in the schema markup
            $markup['offers'] = array(
                '@type' => 'Offer',
                'price' => wc_format_decimal($variation->get_price(), wc_get_price_decimals()),
                'priceCurrency' => get_woocommerce_currency(),
                'availability' => $variation->is_in_stock() ? 'http://schema.org/InStock' : 'http://schema.org/OutOfStock',
                'url' => get_permalink($variation->get_id()),
            );
        }
    }

    return $markup;
}

// Find the matching product variation ID
function qweb_find_matching_product_variation_id( $product ) {
    $attributes = $product->get_attributes();
    $found_variation_id = null;

    foreach ( $_GET as $key => $value ) {
        if ( strpos( $key, 'attribute_' ) === 0 ) {
            $attribute_name = str_replace( 'attribute_', '', $key );
            $attribute_value = sanitize_title( $value );

            if ( array_key_exists( $attribute_name, $attributes ) ) {
                foreach ( $product->get_available_variations() as $variation ) {
                    if ( $variation['attributes'][ 'attribute_' . $attribute_name ] == $attribute_value ) {
                        $found_variation_id = $variation['variation_id'];
                    }
                }
            }
        }
    }

    return $found_variation_id;
}